/*
 *  $Id: map.c,v 1.6 1999/06/11 13:46:33 gregm Exp $
 *
 *  GXSNMP -- An snmp management application
 *  Copyright (c) 1998 Gregory McLean
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc.,  59 Temple Place - Suite 330, Cambridge, MA 02139, USA.
 */

/* 
 * map.c containes utility functions related to the creation and destruction
 * of DB_map items, and functions to manage the DB_map database.
 */
#include <config.h>

#include <gnome.h>
#include "g_sqldb.h"
#include "tables.h"
#include "db_interface.h"
#include "map.h"
#include "graph.h"

#define DEBUG 1
#include "debug.h"

/****************************************************************************
 * map_create
 ***************************************************************************/
DB_map *
map_create (void)
{
  DB_map *dbm;
  DPRT ("map_create (start)");
  dbm           = g_new0 (DB_map, 1);
  if (dbm)
    {
      DPRT ("map_create: memory allocated");
      dbm->created  = g_strdup (_("(not in database)"));
      dbm->modified = g_strdup (dbm->created);
      DPRT ("map_create: returning valid dbm structure.");
      return dbm;
    }
  DPRT ("map_create: returning a NULL dbm structure.");
  g_warning ("Memory allocation failed. expect problems.\n");
  return NULL;
}
/****************************************************************************
 * map_destroy
 ***************************************************************************/
void 
map_destroy (DB_map *dbm)
{
  DPRT ("map_destroy (start)");

  g_assert (dbm->rowid == 0);
  if (dbm)
    {
      DPRT ("map_destroy: freeing data");
      if (dbm->created)        g_free (dbm->created);
      if (dbm->modified)       g_free (dbm->modified);
      if (dbm->name)           g_free (dbm->name);
      if (dbm->tab)            g_free (dbm->tab);
      if (dbm->description)    g_free (dbm->description);
      DPRT ("map_destroy: freeing structure");
      g_free (dbm);
      dbm = NULL;
    }
  DPRT ("map_destroy (end)");
}
/****************************************************************************
 * map_add ()
 ***************************************************************************/
void
map_add (DB_map *dbm)
{
  DPRT ("map_add (start)");
  if (dbm->rowid != 0)
    {
      g_warning ("Attempted to re add a map which is in the DB already.");
      return ;
    }
  dbm->rowid = g_sqldb_highest_rowid (map_sqldb, "_rowid") + 1;
  if (dbm->created) 
    g_free (dbm->created);
  dbm->created = db_timestamp ();
  if (dbm->modified) 
    g_free (dbm->modified);
  dbm->modified = g_strdup (dbm->created);

  g_sqldb_row_add (map_sqldb, dbm);
  DPRT ("map_add (end)");
}
/****************************************************************************
 * map_update 
 ***************************************************************************/
void 
map_update (DB_map *dbm)
{
  DPRT ("map_update (start)");
  g_return_if_fail (dbm->rowid != 0);
  if (dbm->modified) 
    g_free (dbm->modified);
  dbm->modified = db_timestamp ();
  g_sqldb_row_update (map_sqldb, dbm);
  DPRT ("map_update (end");
}
/******************************************************************************
**
**  Function to delete a DB_map object from the database.  The rowid must
**  be non-zero, meaning that the item is already in the database.
**
**  When we delete a DB_map object from the database, we must also delete
**  any DB_graph entries which describe objects on this map.
**
**  When we finished, we set the rowid to zero, meaning that the item is
**  no longer in the database, and return.  The caller is responsible for
**  making a call to map_destroy to free the memory associated with the
**  DB_map block.
**
******************************************************************************/
void
map_delete (DB_map * dbm)
{
  GList        * gl;

  DPRT ("map_delete (start)");
  g_return_if_fail (dbm->rowid != 0);
  DPRT ("map_delete: deleteing graph objects");
  while ((gl = dbm->DB_graphs))         /* Assignment intended */
    {
      graph_delete  ((DB_graph *) gl->data);
      graph_destroy ((DB_graph *) gl->data);
    }
  g_slist_free (dbm->DB_graphs);
  dbm->DB_graphs = NULL;
  g_assert (g_sqldb_row_delete (map_sqldb, dbm) == TRUE);

  dbm->rowid = 0;
  DPRT ("map_delete (end)");
}
/****************************************************************************
 * map_list ()
 ***************************************************************************/
GList *
map_list (void)
{
  return g_sqldb_table_list (map_sqldb);
}
/****************************************************************************
 * map_find_by_rowid ()
 ***************************************************************************/
DB_map *
map_find_by_rowid (guint rowid)
{
  return g_sqldb_row_find (map_sqldb, "_rowid", &rowid);
}
/* EOF */

